{------------------------------------------------------------------------------}
{ Auteur :Martin Beaudet                                                       }
{   Date :26 Janvier 2001                                                      }
{                                                                              }
{   E-Mail : - eraquila@bigfoot.com                                            }
{            - erakis@videotron.ca                                             }
{                                                                              }
{   Page web : - Http://www.citeweb.net/eraquila                               }
{              - Http://www.iquebec.com/eraquila                               }
{              - Http://www.developpez.com                                     }
{                                                                              }
{   Objectif :Crer une classe permettant de charger un bitmap en mmoire.     }
{             De plus, possibilit d'effectuer des traitements sur l'image.    }
{                                                                              }
{                                                                              }
{   Class TFichierBitmap                                                       }
{                                                                              }
{        Constructeur           : Creer;                                       }
{        Destructeur            : Detruire;                                    }
{                                                                              }
{        Chargement d'un Bitmap : Charger( Chemin );                           }
{        Creer un Handle        : CreerHandleBitmap;                           }
{        Creer la palette       : CreerLaPalette;                              }
{        Enregistrer la palette : EnregistrerPalette( Chemin );                }
{        Charger une palette    : ChargerPalette( Chemin );                    }
{                                                                              }
{        Copy de la Class       : CopyBitmapPourEssaie( TFichierBitmap );      }
{                                                                              }
{        Dessiner le Bitmap     : DessinerSur( Hdc, Largeur, Hauteur );        }
{        Dessiner la palette    : DessinerPaletteSur( Hdc, Largeur, Hauteur ); }
{                                                                              }
{        Renverser le bitmap    : FlipHautBas;                                 }
{                                 FlipGaucheDroit;                             }
{        Ajuster la couleur     : AjusterRGB( %R, %V, %B);                     }
{        Inverser les couleurs  : Negatif;                                     }
{        Griser le bitmap       : Griser( %G );                                }
{                                                                              }
{        Conversions :                                                         }
{              1bpp  24bpp                                                    }
{              4bpp  24bpp                                                    }
{              8bpp  24bpp                                                    }
{             16bpp  24bpp                                                    }
{             32bpp  24bpp     :ConvertirEn24bpp                              }
{                                                                              }
{              1bpp  32bpp                                                    }
{              4bpp  32bpp                                                    }
{              8bpp  32bpp                                                    }
{             16bpp  32bpp                                                    }
{             32bpp  32bpp     :ConvertirEn32bpp                              }
{                                                                              }
{                                                                              }
{        Proprits :                                                          }
{             Largeur                                                          }
{             Hauteur                                                          }
{             NbBitsParPixel                                                   }
{             PalNbCouleurs                                                    }
{             DimTableauPixels                                                 }
{             BmpPalette (HPALETTE)                                            }
{             HBmp (HBITMAP)                                                   }
{             pPixels (Pointeur sur Pixels)                                    }
{             PtrDebutPalette (Pointeur sur la palette)                        }
{------------------------------------------------------------------------------}
Unit CLbmp;

Interface
 Uses
  Windows, SysUtils, Classes, Graphics;

{---------- Classe FichierBitmap ----------}
 Type
     TFichierBitmap = Class(TObject)
        Public
         {Proprits du Bitmap}
          Largeur          :Integer;
          Hauteur          :Integer;
          NbBitsParPixel   :Byte;
          PalNbCouleurs    :Cardinal;
          DimTableauPixels :Cardinal;

          BmpPalette       :HPALETTE;
          HBmp             :HBITMAP;
          pPixels          :Pointer;
          PtrDebutPal      :Pointer;
          BitmapCharger    :Boolean; 

         {Constructeur et destructeur du Bitmap}
          Constructor Creer;
          Destructor Detruire;

          {Chargement/Enregistrement du bitmap et des ses composants du Bitmap}
          Procedure Charger (Chemin :String );
          Function CreerHandleBitmap :Boolean;
          Function CreerLaPalette :Boolean;
          Procedure EnregistrerPalette (Chemin :String );
          Procedure ChargerPalette (Chemin :String );
          Function EnregistrerBitmap (Chemin :String ) :Boolean;

          {Copy de Class TFichierBitmap}
          Procedure CopyBitmapClass( TempBitmap :TFichierBitmap );

          {Afficher le bitmap/palette}
          Function DessinerSur( PlancheADessin :HDC;
                                DestLargeur,
                                DestHauteur :Integer )
                               :Boolean;
          Function DessinerPaletteSur( PlancheADessin :HDC;
                                       DestLargeur,
                                       DestHauteur :Integer ) :Boolean;
          {Traitement sur le Bitmap}
          Procedure FlipHautBas;
          Procedure FlipGaucheDroite;
          Procedure AjusterRGB( PourcR, PourcV, PourcB :ShortInt );
          Procedure Negatif;
          Procedure Griser( PourcentClarity :Byte );

          {Conversion de format}
          Procedure ConvertirEn24bpp;
          Procedure ConvertirEn32bpp( Alpha :Byte );

        Private
          BmpINFOFichier     :TBITMAPFILEHEADER;
          BmpINFO            :TBITMAPINFOHEADER;

          LongueurDuneLigne  :Integer;

          GrosseurPal        :Cardinal;

          DimBitmapInfo      :Cardinal;
          pBmpINFO           :PBITMAPINFO;

          pTabPixels         :PByte;

          BitFieldBitmap     :Boolean;
          MasqueRouge,
          MasqueVert,
          MasqueBleu         :DWord;
          Procedure RecupBitFieldMasque;
          Function RecupPos1BitAllumer( MasqueCouleur :DWord) :Byte;
          Function RecupNbBitsAllumer( MasqueCouleur :DWord) :Byte;
     End;
{------------------------------------------}
Implementation
 Uses Unit1;


Constructor TFichierBitmap.Creer;
{Objectif :Constructeur de la class.
           Spcifier qu'aucun fichier bitmap n'est prsentement charg.}
Begin
     Inherited Create;
     BitmapCharger := False;
End;

Procedure TFichierBitmap.Charger (Chemin :String );
{Objectif :Constructeur de la class FichierBitmap. Celle-ci charge un bitmap
           en mmoire et rcupre toutes les infos relatives  celui-ci.}
Var
   BmpFichier     :File;              {Fichier Bitmap}

Begin
     {Spcifier qu'aucun fichier bitmap n'est prsentement charg.}
     BitmapCharger := False;

     {Vrification de l'existance du fichier}
     If Not FileExists( Chemin ) Then
     Begin
          MessageBox( 0, PChar( 'Fichier innexistant.' ),
                     'CLbmp', MB_OK Or MB_ICONSTOP );
          Exit;
     End;

     {Librer la mmoire si un bitmap est dj charg}
     If Assigned( pBmpINFO ) Then
     Begin
          FreeMem( pBmpINFO );
          pBmpINFO := Nil;
     End;
     If Assigned( pTabPixels ) Then
     Begin
          FreeMem( pTabPixels );
          pTabPixels := Nil;
     End;

     {Ouverture du fichier}
     AssignFile( BmpFichier, Chemin );
     Reset( BmpFichier, 1);

     {Lecture de l'entte TBITMAPFILEHEADER.
      Vrifier si c'est bien un bitmap "BM".}
     BlockRead( BmpFichier, BmpINFOFichier, SizeOf(BmpINFOFichier) );
     If BmpINFOFichier.bfType <> ( Ord('M') Shl 8 + Ord('B') ) Then
     Begin
          MessageBox( 0, PChar( 'Mauvais format de bitmap.' ),
                     'CLbmp', MB_OK Or MB_ICONSTOP );
          CloseFile( BmpFichier );
          Exit;
     End;

     {Lecture de l'entte TBITMAPINFO.
     Vrifier si la dimension de l'entte est correct.}
     BlockRead( BmpFichier, BmpINFO, SizeOf(BmpINFO) );
     If BmpINFO.biSize <> SizeOf(BmpINFO) Then
     Begin
          MessageBox( 0, PChar( 'Mauvaise version des enttes.' ),
                     'CLbmp', MB_OK Or MB_ICONSTOP );
          CloseFile( BmpFichier );
          Exit;
     End;

     {Cette classe n'accepte que les fichier Bitmap avec compression RGB
      ou BI_BITFIELD.}
     If (BmpINFO.biCompression <> BI_RGB ) And
        (BmpINFO.biCompression <> BI_BITFIELDS ) Then
     Begin
          MessageBox( 0, PChar( 'Ce type de compression n''est pas gr..' ),
                     'CLbmp', MB_OK Or MB_ICONSTOP );
          CloseFile( BmpFichier );
          Exit;
     End;

     {Dterminer le nombre de couleurs dans la palette}
     If BmpINFO.biClrUsed = 0 Then
        If BmpINFO.biBitCount <= 8 Then
           PalNbCouleurs := 1 Shl BmpINFO.biBitCount
        Else
            PalNbCouleurs := 0
     Else
         PalNbCouleurs := BmpINFO.biClrUsed;

    {Vrifier si le bitmap 16 ou 32bits  une compression BI_BITFIELDS}
     If (BmpINFO.biBitCount = 16) Or (BmpINFO.biBitCount = 32) Then
        If BmpINFO.biCompression = BI_RGB then
           BitFieldBitmap := False
        Else
            If BmpINFO.biCompression = BI_BITFIELDS then
            Begin
                 BitFieldBitmap := True;
                 PalNbCouleurs  := 3;
            End;

     {Dterminer la grosseur d'une palette}
      GrosseurPal := PalNbCouleurs * SizeOf( TRGBQUAD );

     {Dterminer la dimension de la table de couleur ainsi que la structure
      BITMAPINFOHEADER pour enfin connatre la vraie dimension de la structure
          TBITMAPINFO + La table de couleur (TRGBQUAD)
     }
      DimBitmapInfo := Sizeof( TBITMAPINFOHEADER ) + GrosseurPal;

     {Trouv le nombre de quartet d'octet (4 octets conscutif) en largeur }
      LongueurDuneLigne := (((((BmpINFO.biWidth * BmpINFO.biBitCount)+ 7) And $FFFFFFF8) Shr 3) + 3 ) And $FFFFFFFC;

     {Dterminer o dbute le tableau de pixel dans le fichier bitmap.
      On aurait pu aussi utiliser cette ligne qui calcul l'"offset" dplacement
      entre la structure TBITMAPINFO et la fin du fichier, dont ce qui nous
      donnes les donns pixel. Ne pas oublier que TBITMAPINFO contient aussi
      un tableau de couleur pour la palette (Si existante)

         DimTableauPixel := BmpINFOFichier.bfSize - BmpINFOFichier.bfOffBits;
     }
      DimTableauPixels := BmpINFO.biHeight * LongueurDuneLigne;

     {Allocation de mmoire pour rcuprer la structure : "TBITMAPINFO"
          BITMAPINFOHEADER + Table de couleur (TRGBQUAD).}
      GetMem( pBmpINFO, DimBitmapInfo );

     {Copier la structure BITMAPINFOHEADER  l'intrieur de TBITMAPINFO}
      CopyMemory( pBmpINFO, @BmpInfo, SizeOf(TBITMAPINFOHEADER) );

     {Ajouter la palette(TRGBQUAD)  la structure TBITMAPINFO.}
      PtrDebutPal := Pointer( Cardinal(pBmpINFO) + SizeOf(TBITMAPINFOHEADER) );
      BlockRead( BmpFichier, TBITMAPINFO( PtrDebutPal^ ), GrosseurPal );

     {Allouer de la mmoire pour rcuprer les pixels formant l'image dans le bitmap.}
      GetMem( pTabPixels, DimTableauPixels );

     {Positionner le curseur de lecture du fichier, sur le dbut du
      tableau de pixels}
      Seek( BmpFichier, BmpINFOFichier.bfOffBits );

     {Rcuprer le tableau de pixels en mmoire.
      Rcupration du tableau de pixel Original}
      BlockRead( BmpFichier, pTabPixels^, DimTableauPixels );

     {Rcupr un pointeur publique sur les pixel pour des ventuelles
      traitements sur les pixels du Bitmap mais hors de ce unit}
      pPixels := pTabPixels;

     {Fermeture du fichier}
      CloseFile( BmpFichier );

     {Rcuprer les masques si le bitmap est 16bpp
      ou 32bpp et si (Compression = BI_BITFIELD).
      PalNbCouleurs contenant 12 octets mais ce n'tait que pour rcuprer
      les masques de couleurs, bref on le remet  zro car un 16bpp n'a pas
      de palette.}
      If BitFieldBitmap Then
      Begin
           RecupBitFieldMasque;
           PalNbCouleurs := 0;
           GrosseurPal := 0;
      End;
         
     {Crer un Handle sur le bitmap}
      CreerHandleBitmap;

     {Crer la palette}
      CreerLaPalette;

     {Prcision des proprits du bitmap}
      Largeur := BmpINFO.biWidth;
      Hauteur := BmpINFO.biHeight;
      NbBitsParPixel := BmpINFO.biBitCount;

     {Fichier charg correctement}
      BitmapCharger := True;
End;


Destructor TFichierBitmap.Detruire;
{Objectif :Desctructeur de la class FichierBitmap. Toutes les variables dont
           la mmoire  t aloue dynamiquement est libre.}
Begin
     DeleteObject( BmpPalette );
     FreeMem( pTabPixels, DimTableauPixels );
     FreeMem( pBmpInfo, DimBitmapInfo + GrosseurPal );

     Inherited Destroy; {Hritage de la classe TObject}
End;

Procedure TFichierBitmap.RecupBitFieldMasque;
{Objectif :Rcuprer les masques de couleurs si le bitmap est
           compress BI_BITFIELD. L'ordre de rcupration est la suivante :
                #1 - Bleu
                #2 - Vert
                #3 - Rouge
           }
Var
   PtrPal :Pointer;  {Pointeur sur la palette}
Begin
     ptrPal := Pointer( Cardinal( PtrDebutPal ) );

     MasqueRouge := DWord(ptrPal^);
     Inc( Cardinal(ptrPal), SizeOf( DWORD ));
     MasqueVert  := DWord(ptrPal^);
     Inc( Cardinal(ptrPal), SizeOf( DWORD ));
     MasqueBleu  := DWord(ptrPal^);
End;

Function TFichierBitmap.RecupPos1BitAllumer( MasqueCouleur :DWord ) :Byte;
{Objectif :Rcuprer la position dans le DWord o dbute le premier bits
           allum. (Partant de la droite)

           Exemple : 0000011111000000
                              |__________ Donc la 7ime position
           }
Var
   I :Integer;  {Parcourir le DWord}
Begin
     {Parcourir un chaque Bits d'un DWord}
     For I := 0 To SizeOf(DWord) * 8 Do
     Begin
          {Si le bit est allum...}
          If (MasqueCouleur And 1) = 1 Then
          Begin
               {Retourner la position dans le DWord}
               Result := I;
               Exit;
          End;

         {Tester le bit suivant}
          MasqueCouleur := MasqueCouleur Shr 1;
     End;
End;

Function TFichierBitmap.RecupNbBitsAllumer( MasqueCouleur :DWord) :Byte;
{Objectif :Rcuprer le nombre de bits allum dans le masque DWord

           Exemple : 0000011111000000
                          |______________ Ici ce serait 5
           }
Var
   Nb :Integer;  {Parcourir le DWord}
Begin
     Nb := 0;
     While MasqueCouleur <> 0 Do
     Begin
          {Incrmenter de 1 si le bits est allum sinon de 0}
          Inc( Nb, (MasqueCouleur And 1));
          {Tester le bits suivant}
          MasqueCouleur := MasqueCouleur Shr 1;
     End;
     {Renvoyer le rsultat}
     Result := Nb;
End;


Function TFichierBitmap.CreerHandleBitmap :Boolean;
{Objectif :Crer un Handle sur un Bitmap. Cela pourrait tre utile si vous
           dsirez l'affecter  un composant TImage.

           Ex :Image1.Picture.Bitmap.Handle := MonFichierBitmap.hBMP}
Var
   DCTemp :HDC;   {DC temporaire}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
     Begin
          Result := False;
          Exit;
     End;

     {Rcupration du DC de l'cran}
     DCTemp := GetDC( 0 );

     {Cration du Handle sur le bitmap}
     hBmp := CreateDIBitmap( DCTemp,
                             pBmpINFO^.bmiHeader,
                             CBM_INIT,
                             PChar(pTabPixels),
                             pBmpINFO^,
                             DIB_RGB_COLORS);

     {On relche le DC de l'cran (Trs important)}
     ReleaseDC( 0, DCTemp );
     {La cration du bitmap  fonctionn si le rsultat de
      CreateDIBitmap est diffrent de 0}
     Result := hBmp <> 0;
End;

Function TFichierBitmap.CreerLaPalette :Boolean;
{Objectif :Cration d'un Handle sur la palette du Bitmap.
           Comme la cration du Handle sur le Bitmap cela peut s'avrer utile
           si nous dsirons affecter notre palette  un contexte graphique. Ex:

              Image1.Picture.Bitmap.Palette := BmpPalette
           }
Var
   pPaletteTemp :PLOGPALETTE;      {La palette entire ainsi que ses entres}
   NbCouleurs   :Integer;          {Le nombre de couleurs dans la palette}
   PointeurTemp :Pointer;          {Pointer sur notre table de couleur en
                                    mmoire}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
     Begin
          Result := False;
          Exit;
     End;

     {Le Handle sur la palette n'est pas encore cr}
     Result := False;
     BmpPalette := 0;

     {Vrifier si le Bitmap possde bien un Palette}
     If (BmpINFO.biBitCount <= 8) Then
     Begin
          {Allocation de mmoire pour rcuprer toutes les entres de la palette}
          GetMem( pPaletteTemp, SizeOf(TLogPalette) + (PalNbCouleurs * SizeOf( TPaletteEntry )) );

          {La version de la palette (Toujours $300) ainsi que le nombre de
           couleurs dans la palette.}
          TLOGPALETTE(pPaletteTemp^).palVersion := $300;
          TLOGPALETTE(pPaletteTemp^).palNumEntries := PalNbCouleurs;

          {Parcours de notre table de couleur situ en mmoire pour rcuprer
          chaque couleur et les stocker dans notre nouvelle Table de couleur}
          For NbCouleurs := 0 To PalNbCouleurs - 1 do
          Begin
               {Pointeur sur la une couleur de la table de couleur}
               PointeurTemp := Pointer( Integer( PtrDebutPal ) + NbCouleurs * SizeOf(TRGBQuad) );

               {Affectation de la couleur  la nouvelle palette.  l'aide d'un
                transtypage TRGBQuad car une palette contient 4 membres (octets)} 
               TLOGPALETTE(pPaletteTemp^).PalPalEntry[NbCouleurs].peRed   := TRGBQuad(PointeurTemp^).rgbRed;
               TLOGPALETTE(pPaletteTemp^).PalPalEntry[NbCouleurs].peGreen := TRGBQuad(PointeurTemp^).rgbGreen;
               TLOGPALETTE(pPaletteTemp^).PalPalEntry[NbCouleurs].peBlue  := TRGBQuad(PointeurTemp^).rgbBlue;
          End;

          {Cration du Handle sur la nouvelle palette}
          BmpPalette := CreatePalette( TLOGPALETTE(pPaletteTemp^) );

          {On libre la mmoire prise par les entres de couleur stockes
          temporairement}
          FreeMem( pPaletteTemp, SizeOf(TLogPalette) + (PalNbCouleurs * SizeOf( TPaletteEntry)) );

          {Si le Handle sur la palette  t cr alors BmpPalette <> 0}
          Result := BmpPalette <> 0;
     End;
End;


Procedure TFichierBitmap.EnregistrerPalette (Chemin :String );
{Objectif :Enregistrer la palette sous format *.Pal}
Var
   FichierPal :TextFile;  {Fichier pour la palette}
   I          :Integer;   {Incrmenteur pour parcourir la Palette}
   ptrPal     :Pointer;   {Pointeur sur la palette du Bitmap}
begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Seulement les bitmap 8bpp ou moins}
     If BmpINFO.biBitCount <= 8 Then
     Begin
          {Ouverture et cration du fichier}
          AssignFile( FichierPal, Chemin );
          Rewrite( FichierPal );

          {criture de l'auteur et du nombre de couleurs  stockes}
          Writeln( FichierPal, 'Martin Beaudet [CLbmp]' );
          Writeln( FichierPal, IntToStr(PalNbCouleurs) );

          {Inscription des couleurs dans le fichier palette}
          For I := 0 To PalNbCouleurs - 1 Do
          Begin
               ptrPal := Pointer( Integer( PtrDebutPal ) + I * SizeOf( TRGBQUAD ) );
               Writeln( FichierPal, TRGBQUAD( ptrPal^ ).rgbRed,
                                    '  ',
                                    TRGBQUAD( ptrPal^ ).rgbGreen,
                                    '  ',
                                    TRGBQUAD( ptrPal^ ).rgbBlue );
          End;
          {Fermeture du fichier}
          CloseFile( FichierPal );
     End
     Else
         MessageBox( 0, PChar( 'Ce bitmap ne possde pas de palette.' ),
                    'CLbmp', MB_OK Or MB_ICONSTOP );
end;

Procedure TFichierBitmap.ChargerPalette (Chemin :String);
{Objectif :Charger une palette pour le bitmap
           La palette sera charger selon le type donne en paramtre.}
Var
   FichierPal   :TextFile;                 {Fichier pour la palette}
   NamePal      :String;                   {Auteur de la palette}
   NbColor      :Cardinal;                 {Nombre de couleurs}
   ptrPal       :Pointer;                  {Pointeur sur la palette du Bitmap}
   I            :Integer;                  {Incrmenteur pour parcourir la
                                            Palette}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Juste les 8bpp et moins}
     If BmpINFO.biBitCount <= 8 Then
     Begin
          {Ouverture du fichier}
          If Not FileExists( Chemin ) Then
          Begin
              MessageBox( 0, PChar( 'Fichier inexistant.' ),
                         'CLbmp', MB_OK Or MB_ICONSTOP );
              Exit;
          End;

          {Ouverture du fichier *.Clb}
          AssignFile( FichierPal, Chemin );
          Reset( FichierPal );

          {Lecture de l'auteur de la palette}
          Readln( FichierPal, NamePal );
          If UpperCase(NamePal) <> UpperCase('Martin Beaudet [CLbmp]') Then
          Begin
              MessageBox( 0, PChar( 'Format de palette non-reconnue.' ),
                         'CLbmp', MB_OK Or MB_ICONSTOP );
              CloseFile( FichierPal );
              Exit;
          End;

          {Lecture du nombre de couleurs inscrites}
          Readln( FichierPal, NbColor );
          If NbColor = 0 Then
          Begin
              MessageBox( 0, PChar( 'Format de palette non-reconnue.' ),
                         'CLbmp', MB_OK Or MB_ICONSTOP );
              CloseFile( FichierPal );
              Exit;
          End;

          {Vrification si le nombre de couleur inclue dans le fichier palette
           est le mme que celui du Bitmap en cours ou encore si cela
           pourrait fonctionner.}
          If (NbColor > PalNbCouleurs) Then
          Begin
              MessageBox( 0, PChar( 'Taille de palette incompatible.' ),
                         'CLbmp', MB_OK Or MB_ICONSTOP );
              CloseFile( FichierPal );
              Exit;
          End;

         {Rcupration de la deuxime palette}
          For I := 0 To PalNbCouleurs - 1 Do
          Begin
               ptrPal := Pointer( Integer( PtrDebutPal ) + I * SizeOf( TRGBQUAD ) );

              {Remplacement directe des valeur de la palette
               original par celles d'une autre palette de
               mme dimension.}
               Readln( FichierPal, TRGBQUAD( ptrPal^ ).rgbRed,
                                   TRGBQUAD( ptrPal^ ).rgbGreen,
                                   TRGBQUAD( ptrPal^ ).rgbBlue );
                                   TRGBQUAD( ptrPal^ ).rgbReserved := PC_RESERVED;
          End;
          {Fermeture du fichier}
          CloseFile( FichierPal );

          {Recrer la palette}
          CreerLaPalette;
     End
     Else
         MessageBox( 0, PChar( 'Ce Bitmap ne peut possder de Palette.' ),
                    'CLbmp', MB_OK Or MB_ICONSTOP );
end;

Function TFichierBitmap.EnregistrerBitmap (Chemin :String ) :Boolean;
{Objectif :Enregistrer le bitmap sur disque}
Var
   BmpFichier     :File;              {Fichier Bitmap}
   TailleBitmap   :Integer;           {Taille du fichier}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
     Begin
          Result := False;
          Exit;
     End;

     {Aucun fichier ouvert}
     Result := False;

     Try
        {Dterminer la taille du fichier  crer}
        TailleBitmap := SizeOf( TBITMAPFILEHEADER ) + SizeOf(TBITMAPINFOHEADER)
                        + GrosseurPal + DimTableauPixels;

        {S'il sagit d'un Bitmap avec le type de conpression BI_BITFIELD alors
         rserv 3 DWord supplmentaire pour les masques}
         If BitFieldBitmap Then
            Inc(TailleBitmap, 3 * SizeOf(Dword) );

        {Cration du fichier}
        AssignFile( BmpFichier, Chemin );
        ReWrite( BmpFichier, TailleBitmap );
        Reset( BmpFichier, 1);

        {criture de l'entte de fichier Bitmap}
        BlockWrite( BmpFichier, TBITMAPFILEHEADER(BmpINFOFichier), SizeOf( BmpINFOFichier ) );

        {criture de l'entte d'info du Bitmap}
        BlockWrite( BmpFichier, TBITMAPINFO(pBmpInfo^), SizeOf(TBITMAPINFOHEADER) );

        {S'il sagit d'un Bitmap avec le type de conpression BI_BITFIELD alors
         lire les 3 masques de couleurs}
         If BitFieldBitmap Then
         Begin
              BlockWrite( BmpFichier, MasqueRouge, SizeOf( DWord) );
              BlockWrite( BmpFichier, MasqueVert, SizeOf( DWord) );
              BlockWrite( BmpFichier, MasqueBleu, SizeOf( DWord) );
         End;

         {crire la palette. Si aucune palette alors GrosseurPal = 0}
         BlockWrite( BmpFichier, TRGBQuad(PtrDebutPal^), GrosseurPal );

         {crire les pixels du Bitmap}
         BlockWrite( BmpFichier, Byte(pTabPixels^), DimTableauPixels );

         {Fermeture du fichier}
         CloseFile( BmpFichier );

         {Fichier crite avec succs}
         Result := True;

     Except
           On EInOutError Do
           Begin
                MessageBox( 0, PChar('Erreur de lecture'), Pchar('CLbmp'), MB_ICONSTOP + MB_OK );
                Result := False;
           End;
     End;
End;


Procedure TFichierBitmap.CopyBitmapClass( TempBitmap :TFichierBitmap );
{Objectif :Effectuer une copie de class TFichierBitmap.
           Cela peut-tre utile pour effectuer un traitement temporaire sur le
           un bitmap et ensuite l'appliquer s'il correspond  nos besions.}
Begin
     {Vrification de l'existance de la class pass en paramtre}
     If TempBitmap = Nil Then
     Begin
          MessageBox( 0, PChar( 'Erreur, Bitmap on existant en mmoire.' ),
                     'CLbmp', MB_OK Or MB_ICONSTOP );
          Exit;
     End;

     {Rcupration des variables statiques}
     Largeur := TempBitmap.Largeur;
     Hauteur := TempBitmap.Hauteur;
     NbBitsParPixel := TempBitmap.NbBitsParPixel;
     PalNbCouleurs := TempBitmap.PalNbCouleurs;
     DimTableauPixels := TempBitmap.DimTableauPixels;
     BmpINFOFichier := TempBitmap.BmpINFOFichier;
     BmpINFO := TempBitmap.BmpINFO;
     LongueurDuneLigne := TempBitmap.LongueurDuneLigne;
     GrosseurPal := TempBitmap.GrosseurPal;
     DimBitmapInfo := TempBitmap.DimBitmapInfo;
     BitFieldBitmap := TempBitmap.BitFieldBitmap;
     MasqueRouge := TempBitmap.MasqueRouge;
     MasqueVert := TempBitmap.MasqueVert;
     MasqueBleu := TempBitmap.MasqueBleu;

     {Copie des structures (Pointeurs)}
     GetMem( pBmpINFO, DimBitmapInfo );
     CopyMemory( pBmpINFO, @TempBitmap.BmpInfo, SizeOf(TBITMAPINFOHEADER) );

     PtrDebutPal := Pointer( Cardinal(pBmpINFO) + SizeOf(TBITMAPINFOHEADER) );
     {Si c'est un format compress BI_BITFIELD alors prsumer que la palette
     contient les 3 masques (DWORD) de couleur}
     If BitFieldBitmap Then
      CopyMemory( PBITMAPINFO(PtrDebutPal), TempBitmap.PtrDebutPal, 3 * SizeOf(DWord) )
     Else
         CopyMemory( PBITMAPINFO(PtrDebutPal), TempBitmap.PtrDebutPal, GrosseurPal );

     GetMem( pTabPixels, DimTableauPixels );
     CopyMemory( pTabPixels, TempBitmap.pTabPixels, DimTableauPixels );

     pPixels := pTabPixels;

     {Cration des objects}
     CreerHandleBitmap;
     CreerLaPalette;
End;


Function TFichierBitmap.DessinerSur( PlancheADessin :HDC; DestLargeur,
                                     DestHauteur :Integer ) :Boolean;
{Objectif :Crer un bitmap compatible au contexte graphique (DC) pass en
           paramtre.
           Ensuite crer un DC temporaire o sera dessiner le Bitmap.
           Copier le contenu du DC temporaire dans celui pass en paramtre.}
Var
   hBMP :HBITMAP;        {Bitmap compatible au DC pass en paramtre}
   MemoireDC :HDC;       {DC temporaire compatible  celui pass en paramtre}

Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
     Begin
          Result := False;
          Exit;
     End;

     {Bitmap non dessin}
      Result := False;

     {Vrification du type de DC pass en paramtre}
      If GetObjectType( PlancheADessin ) = 0 Then
      Begin
           MessageBox( 0, PChar( 'Mauvais contexte graphique en paramtre.' ),
                      'CLbmp', MB_OK Or MB_ICONSTOP );
           Exit;
      End;

     {Cration du bitmap compatible et vrification de sa cration}
      hBMP := CreateCompatibleBitmap( PlancheADessin,
                                      BmpINFO.biWidth, BmpINFO.biHeight );
      If hBMP = NULL Then
      Begin
           MessageBox( 0, PChar( 'Erreur  la cration du Bitmap.' ),
                      'CLbmp', MB_OK Or MB_ICONSTOP );
           Exit;
      End;

     {Cration du DC temporaire et vrification de celui-ci}
      MemoireDC := CreateCompatibleDC( PlancheADessin );
      If MemoireDC = NULL Then
      Begin
           MessageBox( 0, PChar( 'Erreur  la cration du contexte ' +
                                 'graphique temporaire.' ),
                      'Clbmp', MB_OK Or MB_ICONSTOP );
           Exit;
      End;

     {Envoie du tableau de pixel vers le DC temporaire et vrification}
      If SetDIBits( MemoireDC, hBMP, 0, BmpINFO.biHeight,
                    pTabPixels, pBmpINFO^, DIB_RGB_COLORS ) <= 0 Then
      Begin
           MessageBox( 0, PChar( 'Erreur  l''affichage du Bitmap.' ),
                      'Clbmp', MB_OK Or MB_ICONSTOP );
           Exit;
      End;

     {Slection du Bitmap sur le DC temporaire}
      SelectObject( MemoireDC , hBMP );

     {Dessine le Bitmap sur le vrai DC pass en pamamre et vrification}
      If StretchBlt( PlancheADessin,
                     0, 0, DestLargeur, DestHauteur,
                     MemoireDC,
                     0, 0, BmpINFO.biWidth, BmpINFO.biHeight,
                     SRCCOPY ) <> False Then
         Result := True ;

     {Supression du Bitmap et du DC temporaire}
      DeleteObject( hBMP );
      DeleteDC( MemoireDC );
End;


Function TFichierBitmap.DessinerPaletteSur( PlancheADessin :HDC;
                                            DestLargeur,
                                            DestHauteur :Integer ) :Boolean;
{Objectif :Cette function  pour but de dessiner la palette du Bitmap sur un
           contexte graphique (DC).}
Var
   NouveauPinceau,                   {Pineau qui dessiner la palette}
   AncienPinceau    :HBrush;         {Ancien pineau}
   Colonnes, Lignes :Integer;        {Colonnes et Lignes de la PlancheADessin}
   NbCouleurs       :Integer;        {Nombre de couleur dans la palette}
   PointeurTemp     :^TRGBQuad;      {Pointeur sur la table de couleur}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
     Begin
          Result := False;
          Exit;
     End;

     {Palette non-dessine}
     Result := False;

     {Vrification du type de DC pass en paramtre}
     If GetObjectType( PlancheADessin ) = 0 Then
     Begin
          MessageBox( 0, PChar( 'Mauvais contexte graphique en paramtre.' ),
                     'Clbmp', MB_OK Or MB_ICONSTOP );
          Exit;
     End;

     If (BmpINFO.bibitCount <= 8) Then
     Begin
          {Parcourir la palette de couleur en couleurs}
          For NbCouleurs := 0 To PalNbCouleurs - 1 Do
          Begin
               Lignes := NbCouleurs Div 16;
               Colonnes := NbCouleurs Mod 16;

               {Pointeur sur une couleur de la table}
               PointeurTemp := Pointer( Integer( PtrDebutPal ) + NbCouleurs * SizeOf(TRGBQuad) );

               {Cration d'un pineau dont sa couleur est celle rcupre  la
               position NbCouleurs de la palette}
               NouveauPinceau := CreateSolidBrush( RGB(
                                                TRGBQuad( PointeurTemp^).rgbRed,
                                                TRGBQuad( PointeurTemp^).rgbGreen,
                                                TRGBQuad( PointeurTemp^).rgbBlue )
                                                 );
               {On affecte ce nouveau pineau  notre PlancheADessin}
               AncienPinceau := SelectObject( PlancheADessin, NouveauPinceau );

               {On dessiner le rectangle contenant la couleur NbCouleurs de
               la palette}
               RoundRect( PlancheADessin,
                          (DestLargeur Div 16) * Colonnes,
                          (DestHauteur Div 16) * Lignes,
                          (DestLargeur Div 16) * (Colonnes + 1),
                          (DestHauteur Div 16) * (Lignes + 1),
                          5, 5 );

               {On reslectionne l'ancien pineau (Important)}
               SelectObject( PlancheADessin, AncienPinceau );
               {Supression du pineau qui  servit  dessiner la palette}
               DeleteObject( NouveauPinceau );
          End;
          {Palette dessiner}
          Result := True;
     End
     Else
         Begin
              MessageBox( 0, PChar( 'Ce bitmap ne possde pas de palette.' ),
                          'Clbmp', MB_OK Or MB_ICONSTOP );
         End;
End;


Procedure TFichierBitmap.FlipHautBas;
{Objectif :Effectuer un symtrie vertical du Bitmap. Le but est de rcuprer
           la chaque lignes horizontal du bitmap et de les inverser.}
Var
   ParcoursAvant,               {Premire...}
   ParcoursArriere  :Integer;   {...et dernire lignes du Bitmap}
   pLignePixelTemp  :PByte;     {Ligne de pixel temporaire pour change}
begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Allocation de la mmoire pour cette ligne}
     GetMem( pLignePixelTemp, LongueurDuneLigne );

     {Parcours du Bitmap de Bas->Haut}
     ParcoursAvant     := 0;
     ParcoursArriere   := LongueurDuneLigne * (BmpINFO.biHeight - 1);

     {Parcours en vertical du Bitmap (Moiti)}
     While ParcoursAvant < LongueurDuneLigne * (BmpINFO.biHeight Div 2) Do
     Begin
          {Rcupration en mmoire d'une ligne}
          CopyMemory( pLignePixelTemp, Pointer( Integer(pTabPixels) + ParcoursAvant), LongueurDuneLigne );
          {On recopie cette ligne  l'oppose}
          CopyMemory( Pointer( Integer(pTabPixels) + ParcoursAvant), Pointer( Integer(pTabPixels) + ParcoursArriere), LongueurDuneLigne );
          {On se sert de la ligne en mmoire pour la recopi  l'oppose}
          CopyMemory( Pointer( Integer(pTabPixels) + ParcoursArriere), pLignePixelTemp, LongueurDuneLigne );

          {On se rapproche du milieu}
          Inc(ParcoursAvant, LongueurDuneLigne );
          Dec(ParcoursArriere, LongueurDuneLigne );
      End;

      {Libre la mmoire prise par la ligne temporaire en mmoire}
      FreeMem( pLignePixelTemp, LongueurDuneLigne);
End;


Function SwapQuartet( I :Byte ) :Byte; Assembler;
{Objectif :Cette function retourne un Byte (Octet) dont chacun de ses bits ont
           subit un rotation de 7 bits vers la gauche. Donc inversion des bits
           dans l'octet reu en paramtre}
Asm
   {Rcupre le paramtre, celui dont les bytes doivent subir une rotation}
   MOV al, I
   {Rotation de "cl" bits vers la gauche}
   ROR al, 4
End;

Function SwapBits( I :Byte ) :Byte; Assembler;
{Objectif :Inverser les bits d'un octet

              Ex : 10010010  -> 01001001
}
Asm
   MOV  DL, I         {Rcuprer dans DL le paramtre I}
   MOV  CX, 8         {Prparer une LOOP de 8 incrmentations}

   @BitFlip:          {tiquette qui sera rpte 8 fois}
            RCL DL,1  {Rotation vers la droite}
            RCR AL,1  {Rotation vers la gauche}
   LOOP @BitFlip      {Retour dans la boucle si non-rpt 8 fois}
End;


Procedure TFichierBitmap.FlipGaucheDroite;
{Objectif :Le but ici est de rcupr chaque ligne horizontal du Bitmap et
           d'invers soit les bits pour un bitmap 1, 4 bits/pixel ou sinon
           on inverse l'ordre des octets. En bout de ligne on obtient un Bitmap
           ayant subit un renversement horizontal.}
Const
  MasqueBits : Array[0..7] Of byte = ($00, $7F, $3F, $1F, $0F, $07, $03, $01);

Var
   PtrBitmapOriginal:pByte;       {Pointeur sur le tableau de pixel du
                                   bitmap original}

   Y                :Integer;     {Hauteur du bitmap parcourue }
   X                :Integer;     {Variable contrle pour parcourir la ligne}

   BitsEnTrop       :Integer;     {Bits en trop sur une ligne}

   pLignePixelTemp  :^Byte;       {Ligne de pixel temporaire pour change}

   ptrByteSource,
   ptrByteCible     :^Byte;       {Pointeur pour change de byte}

   ptrWordSource,
   ptrWordCible     :^Word;       {Pointeur pour change de DWord}

   ptrTripSource,
   ptrTripCible     :^TRGBTriple; {Pointeur pour change de Triple}

   ptrQuadSource,
   ptrQuadCible     :^TRGBQuad;   {Pointeur pour change de Quad}

Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Allocation de la mmoire pour cette ligne}
     GetMem( pLignePixelTemp, LongueurDuneLigne );

     {Rcuprer un pointeur sur le tableau de pixel du bitmap Original}
     PtrBitmapOriginal := pTabPixels;

     {Parcours du Bitmap de Bas->Haut}
     Y := 0;
     While Y < (BmpINFO.biHeight) Do
     Begin
          {On rcupre la ligne horizontal de pixels en mmoire}
          CopyMemory( pLignePixelTemp, PtrBitmapOriginal, LongueurDuneLigne );

          {Traitement diffrent selon le nombre de bits par pixel}
          Case BmpINFO.biBitCount Of

          {BITMAP 1 BITS/PIXELS}
          1 :Begin
                  {Pointeur la ligne source du Bitmap
                   (Shr 3, car 8 pixels = 1 Octets)}
                   ptrByteSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1) Shr 3) );

                  {Pointeur sur la ligne temporaire en mmoire}
                   ptrByteCible := Pointer( Integer(pLignePixelTemp) );

                  {On inverser l'ordre des bits de chaque octets.
                   Si la largeur du Bitmap n'est pas un multiple de 8 alors
                   il y aura des bits qui seront oublis donc repris par
                   les instructions succdant cette boucle.}
                  For X := 0 To ((BmpINFO.biWidth -1) Shr 3) Do
                  Begin
                       ptrByteCible^ := SwapBits( ptrByteSource^ );
                       Inc( Integer(ptrByteCible), 1 );
                       Dec( Integer(ptrByteSource), 1 );
                  End;

                  {Vrifier si le nombre de bits dans une ligne est
                   un multiple de 8. Si non alors reprendre les bits oublis.
                   Ceux-ci seront dcaller  l'extrme gauche du Bitmap.}
                  BitsEnTrop := BmpINFO.biWidth Mod 8;
                  If BitsEnTrop <> 0 Then
                  Begin
                       {Pointeur sur le premier octet de la ligne temporaire}
                       ptrByteCible  := Pointer( Integer(pLignePixelTemp));

                       {Pointeur sur le deuxime octet de la ligne temporaire}
                       ptrByteSource := Pointer( Integer(pLignePixelTemp) + 1);

                       {Dcaller  la gauche du Bitmap, chaque bits de chaque
                        octets rencontrs}
                       For X := 0 To ((BmpINFO.biWidth -1) Shr 3) Do
                       Begin
                            ptrByteCible^ := (ptrByteCible^ Shl (8 - BitsEnTrop) ) Or ( Byte(ptrByteSource^) Shr (BitsEnTrop) ) And MasqueBits[BitsEnTrop];
                            Inc( Integer(ptrByteCible), 1 );
                            Inc( Integer(ptrByteSource), 1 );
                       End;
                   End;
             End;

          {BITMAP 4 BITS/PIXELS}
          4:Begin
                  {Pointeur la ligne source du Bitmap
                   (Shr 1, car 2 pixels = 1 Octets)}
                   ptrByteSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1) Shr 1) );

                  {Pointeur sur la ligne temporaire en mmoire}
                   ptrByteCible := Pointer( Integer(pLignePixelTemp) );

                  {Vrifier si le nombre de bits dans une ligne est
                   un multiple de 2. Si non alors reprendre le quartet oublis.
                   Ceux-ci seront dcaller  l'extrme gauche du Bitmap.
                   Mod 2 car un Octet possde 2 Quartet et par la suite
                   il nous sera utile de savoir lequel il faut choisir entre
                   le le quartet de poid fort et celui de poid faible.}
                  BitsEnTrop := BmpINFO.biWidth Mod 2;
                  If BitsEnTrop <> 0 Then
                  Begin
                       {Dcaller  la gauche du Bitmap, chaque quartets de chaque
                        octets rencontrs}
                       For X := 0 To (BmpINFO.biWidth -1) Do
                       Begin
                            {Vrifier si le quartet de la source est un quartet
                            de poids fort ou faible et ensuite le rcuprer.}
                            If ((BmpINFO.biWidth - X) Mod 2) = 1 Then
                               BitsEnTrop := ptrByteSource^ Shr 4     {Poids fort}
                            Else
                                BitsEnTrop := ptrByteSource^ And $0F; {Poids faible}

                            {Vrifier o est rendu X dans son parcours de la
                             ligne. Ou plutt, vrifier sur quel type de
                             quartet X est arrter (Poids fort ou faible).
                             Ensuite rcrire ce quartet avec les donne
                             compris dans BitsEnTrop}
                            If (X Mod 2) = 0 Then
                               ptrByteCible^ := BitsEnTrop Shl 4     {Poids fort}
                            Else
                                ptrByteCible^ := (ptrByteCible^ And $F0) Or (BitsEnTrop And $0F); {Poids faible}

                            {Si la source a parcourue les deux quartets de poids
                             fort alors passez  au poids faible de l'octets
                             suivant}
                            If ((BmpINFO.biWidth - X) Mod 2) = 1 Then
                               Dec( Integer(ptrByteSource), 1 );

                            {Si X a parcourue les deux quartet de poids fort
                            alors passez  au poids faible de l'octets suivant}
                            If (X Mod 2) = 1 Then
                               Inc( Integer(ptrByteCible), 1 );
                       End;
                   End
                   Else
                       {Si nous sommes ici c'est que le nombre de quartet
                        trouv dans la ligne est un multiple de 2.
                        Ceci implique alors que chaque quartet est utilis pour
                        dessiner le bitmap et qu'il n'y en a pas un en trop.}
                       For X := 0 To ((BmpINFO.biWidth -1) Shr 1) Do
                       Begin
                            ptrByteCible^ := SwapQuartet( ptrByteSource^ );
                            Inc( Integer(ptrByteCible), 1 );
                            Dec( Integer(ptrByteSource), 1 );
                       End;
            End;

          {BITMAP 8 BITS/PIXELS}
          8:Begin
                {Pointeur la ligne source du Bitmap. Le nombre de pixel
                 concorde avec le nombre d'octet.}
                 ptrByteSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1)) );

                {Pointeur sur la ligne temporaire en mmoire}
                 ptrByteCible := Pointer( Integer(pLignePixelTemp) );

                {On lit la source et on affecte la cible  l'envers}
                 For X := 0 To BmpINFO.biWidth - 1 Do
                 Begin
                      ptrByteCible^ := ptrByteSource^;
                      Inc( Integer(ptrByteCible), 1 );
                      Dec( Integer(ptrByteSource), 1 );
                 End;
            End;

          {BITMAP 16 BITS/PIXELS}
         16:Begin
                {Pointeur la ligne source du Bitmap. Il faut 2 octets pour
                 un pixel alors on multiple par 2.}
                 ptrWordSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1)) Shl 1 );

                {Pointeur sur la ligne temporaire en mmoire}
                 ptrWordCible := Pointer( Integer(pLignePixelTemp) );

                {On lit la source et on affecte la cible  l'envers}
                 For X := 0 To BmpINFO.biWidth - 1 Do
                 Begin
                      ptrWordCible^ := ptrWordSource^;
                      Inc( Integer(ptrWordCible), 2 );
                      Dec( Integer(ptrWordSource), 2 );
                 End;
            End;

          {BITMAP 24 BITS/PIXELS}
         24:Begin
                {Pointeur la ligne source du Bitmap. Il faut 3 octets pour
                 un pixel alors on multiple par 3.}
                 ptrTripSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1) * 3 ));

                {Pointeur sur la ligne temporaire en mmoire}
                 ptrTripCible := Pointer( Integer(pLignePixelTemp) );

                {On lit la source et on affecte la cible  l'envers}
                 For X := 0 To BmpINFO.biWidth - 1 Do
                 Begin
                      ptrTripCible^ := ptrTripSource^;
                      Inc( Integer(ptrTripCible), 3 );
                      Dec( Integer(ptrTripSource), 3 );
                 End;
            End;

          {BITMAP 32 BITS/PIXELS}
         32:Begin
                {Pointeur la ligne source du Bitmap. Il faut 4 octets pour
                 un pixel alors on multiple par 8.}
                 ptrQuadSource := Pointer( Integer(PtrBitmapOriginal) + ((BmpINFO.biWidth - 1) * 4) );

                {Pointeur sur la ligne temporaire en mmoire}
                 ptrQuadCible := Pointer( Integer(pLignePixelTemp) );

                {On lit la source et on affecte la cible  l'envers}
                 For X := 0 To BmpINFO.biWidth - 1 Do
                 Begin
                      ptrQuadCible^ := ptrQuadSource^;
                      Inc( Integer(ptrQuadCible), 4 );
                      Dec( Integer(ptrQuadSource), 4 );
                 End;
            End;

          End;

          {On recopie la ligne modifi  l'intrieur du tableau de pixel
           du bitmap}
          CopyMemory( PtrBitmapOriginal, pLignePixelTemp , LongueurDuneLigne );

          {On passe  l'autre ligne}
          Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );

          {On change de ligne, vers le HAUT}
          Inc( Y );
     End;

    {Libre la mmoire prise par la ligne de pixel qui  servit  traiter
     les pixels}
     FreeMem( pLignePixelTemp, LongueurDuneLigne);
End;


Procedure TFichierBitmap.AjusterRGB( PourcR, PourcV, PourcB :ShortInt );
{Objectif :Ajuster en termet de pourcentage chaque couleur du Bitmap.
           Voici la formule permettant ce changement :

                      NouveauRouge = Rouge * ( 1 + Pourcentage / 100 )
                      NouveauVert  = Vert  * ( 1 + Pourcentage / 100 )
                      NouveauBleu  = Bleu  * ( 1 + Pourcentage / 100 )

           Comme les bitmap 1, 4 et 8bpp ne possde pas la description pour
           la couleur de ses pixels mais plutt un index qui rfre  une
           couleur dans un Palette alors pour modifier leur couleur il faudra
           travailler avec leurs palettes. Sinon pour les autres on modifie
           directement les bits qui dcrit la couleur de la pixel.}
Var
   X, Y                   :Integer;  {Parcourir la palette ou les pixels}
   ptrBitmapOriginal      :Pointer;  {Pointeur sur les pixels du bitmap}
   ptrSource              :Pointer;  {Pointeur sur une ligne du bitmap}
   ptrPal                 :Pointer;  {Pointeur sur la palette}
   R, V, B                :Integer;  {Couleurs temporaire pour les 16bpp}
   NbBitRouge,
   NbBitVert,
   NbBitBleu              :Byte;     {Position du premier bits allume}
   NbBitAllumR,
   NbBitAllumV,
   NbBitAllumB            :Byte;     {Nombre de bits conscutif allums}

   NouvelleCouleur        :Integer;  {Couleur temporaire pour les 16bpp}
   CouleurIndex           :Integer;  {Parcourir la palette du Bitmap}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Rcuprer un pointeur sur les Pixel du bitmap}
     PtrBitmapOriginal := pTabPixels;

     {Vrification des bornes}
     If PourcR > 100 Then
        PourcR := 100
     Else
         If PourcR < -100 Then
            PourcR := -100;
     If PourcV > 100 Then
        PourcV := 100
     Else
         If PourcV < -100 Then
            PourcV := -100;
     If PourcB > 100 Then
        PourcB := 100
     Else
         If PourcB < -100 Then
            PourcB := -100;

     {Dpendemment du nombre de bits par Pixel...}
     Case BmpINFO.biBitCount Of
      1, 4, 8:Begin
                {Modifier directement la palette du bitmap}
                For CouleurIndex := 0 To PalNbCouleurs - 1 Do
                Begin
                     ptrPal := Pointer( Integer( PtrDebutPal ) + CouleurIndex * SizeOf(TRGBQuad) );

                     {On applique la formule magique.}
                     B := Round(TRGBQuad(ptrPal^).rgbBlue * ( 1 +  PourcB / 100 ));
                     {Vrification des bornes maximum et minimum d'un Octet}
                     If B >= 255 Then
                        TRGBQuad(ptrPal^).rgbBlue := 255
                     Else
                         If B <= 0 Then
                            TRGBQuad(ptrPal^).rgbBlue := 0
                         Else
                             TRGBQuad(ptrPal^).rgbBlue := B;

                     V := Round(TRGBQuad(ptrPal^).rgbGreen * ( 1 +  PourcV / 100 ));
                     If V >= 255 Then
                        TRGBQuad(ptrPal^).rgbGreen := 255
                     Else
                         If V <= 0 Then
                            TRGBQuad(ptrPal^).rgbGreen := 0
                         Else
                             TRGBQuad(ptrPal^).rgbGreen := V;

                     R := Round(TRGBQuad(ptrPal^).rgbRed * ( 1 +  PourcR / 100 ));
                     If R >= 255 Then
                        TRGBQuad(ptrPal^).rgbRed := 255
                     Else
                         If R <= 0 Then
                            TRGBQuad(ptrPal^).rgbRed := 0
                         Else
                             TRGBQuad(ptrPal^).rgbRed := R;
                End;
            End;

         16:Begin
                 {Rcuprer le nombre de bits conccutif allums}
                 NbBitRouge := RecupNbBitsAllumer( MasqueRouge);
                 NbBitVert  := RecupNbBitsAllumer( MasqueVert);
                 NbBitBleu  := RecupNbBitsAllumer( MasqueBleu);

                 {Rcuprer la position du premier bits allume dans le Masque}
                 NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                 NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                 NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                 Y := 0;
                 While Y < BmpINFO.biHeight Do
                 Begin
                      ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                      For X := 0 To (BmpINFO.biWidth - 1) Do
                      Begin
                           If BitFieldBitmap Then
                           Begin
                                R := (DWord(ptrSource^) And MasqueRouge) Shr NbBitAllumR;
                                V := (DWord(ptrSource^) And MasqueVert) Shr NbBitAllumV;
                                B := (DWord(ptrSource^) And MasqueBleu);

                                {Ajuster le bleu}
                                B := Round( B * (1 + PourcB / 100));
                                If B > (1 Shl NbBitRouge) - 1  Then
                                   B := (1 Shl NbBitRouge) - 1
                                Else
                                    If B < 0 Then
                                       B := 0;
                                {Ajuster le vert}
                                V := Round( V * (1 + PourcV / 100));
                                If V > (1 Shl NbBitVert) - 1 Then
                                   V := (1 Shl NbBitVert) - 1
                                Else
                                    If V < 0 Then
                                       V := 0;
                                {Ajuster le rouge}
                                 R := Round( R * (1 + PourcR / 100));
                                 If R > (1 Shl NbBitBleu) - 1 Then
                                    R := (1 Shl NbBitBleu) - 1
                                 Else
                                     If R < 0 Then
                                        R := 0;

                                NouvelleCouleur := (R Shl NbBitAllumR) Or (V Shl NbBitAllumV) Or B;
                           End
                           Else
                               Begin
                                    R := (DWord(ptrSource^) And $7C00) Shr 10;
                                    V := (DWord(ptrSource^) And $03E0) Shr 5;
                                    B := (DWord(ptrSource^) And $001F);

                                    {Ajuster le bleu}
                                    B := Round( B * (1 + PourcB / 100));
                                    If B > 31 Then
                                       B := 31
                                    Else
                                        If B < 0 Then
                                           B := 0;
                                    {Ajuster le vert}
                                    V := Round( V * (1 + PourcV / 100));
                                    If V > 31 Then
                                       V := 31
                                    Else
                                        If V < 0 Then
                                           V := 0;
                                    {Ajuster le rouge}
                                    R := Round( R * (1 + PourcR / 100));
                                    If R > 31 Then
                                       R := 31
                                    Else
                                        If R < 0 Then
                                           R := 0;

                                     NouvelleCouleur := (R Shl 10) Or (V Shl 5) Or B;
                             End;

                           Word(ptrSource^) := NouvelleCouleur;
                           Inc( Integer(ptrSource), 2 );
                      End;

                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Y );
                 End;
            End;

         24:Begin
                 Y := 0;
                 While Y < BmpINFO.biHeight Do
                 Begin
                      ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                      For X := 0 To (BmpINFO.biWidth - 1) Do
                      Begin
                           B := Round( TRGBTriple(ptrSource^).rgbtBlue * ( 1 +  PourcB / 100 ));
                           If B > 255 Then
                              TRGBTriple(ptrSource^).rgbtBlue := 255
                           Else
                               If B < 0 Then
                                  TRGBTriple(ptrSource^).rgbtBlue := 0
                               Else
                                   TRGBTriple(ptrSource^).rgbtBlue := B;

                           V := Round(TRGBTriple(ptrSource^).rgbtGreen * ( 1 +  PourcV / 100 ));
                           If V > 255 Then
                              TRGBTriple(ptrSource^).rgbtGreen := 255
                           Else
                               If V < 0 Then
                                  TRGBTriple(ptrSource^).rgbtGreen := 0
                               Else
                                   TRGBTriple(ptrSource^).rgbtGreen := V;

                           R := Round(TRGBTriple(ptrSource^).rgbtRed * ( 1 +  PourcR / 100 ));
                           If R > 255 Then
                              TRGBTriple(ptrSource^).rgbtRed := 255
                           Else
                               If R < 0 Then
                                  TRGBTriple(ptrSource^).rgbtRed := 0
                               Else
                                   TRGBTriple(ptrSource^).rgbtRed := R;


                           Inc( Integer(ptrSource), 3 );
                      End;

                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Y );
                 End;
            End;

         32:Begin
                 {Rcuprer le nombre de bits conccutif allums}
                 NbBitRouge := RecupNbBitsAllumer( MasqueRouge);
                 NbBitVert  := RecupNbBitsAllumer( MasqueVert);
                 NbBitBleu  := RecupNbBitsAllumer( MasqueBleu);

                 {Rcuprer la position du premier bits allume dans le Masque}
                 NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                 NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                 NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                 Y := 0;
                 While Y < BmpINFO.biHeight Do
                 Begin
                      ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                      For X := 0 To (BmpINFO.biWidth - 1) Do
                      Begin
                           If BitFieldBitmap Then
                           Begin
                                R := (DWord(ptrSource^) And MasqueRouge) Shr NbBitAllumR;
                                V := (DWord(ptrSource^) And MasqueVert) Shr NbBitAllumV;
                                B := (DWord(ptrSource^) And MasqueBleu);

                                {Ajuster le bleu}
                                B := Round( B * (1 + PourcB / 100));
                                If B > (1 Shl NbBitRouge) - 1  Then
                                   B := (1 Shl NbBitRouge) - 1
                                Else
                                    If B < 0 Then
                                       B := 0;
                                {Ajuster le vert}
                                V := Round( V * (1 + PourcV / 100));
                                If V > (1 Shl NbBitVert) - 1 Then
                                   V := (1 Shl NbBitVert) - 1
                                Else
                                    If V < 0 Then
                                       V := 0;
                                {Ajuster le rouge}
                                 R := Round( R * (1 + PourcR / 100));
                                 If R > (1 Shl NbBitBleu) - 1 Then
                                    R := (1 Shl NbBitBleu) - 1
                                 Else
                                     If R < 0 Then
                                        R := 0;

                                 TRGBQuad(ptrSource^).rgbRed   := R;
                                 TRGBQuad(ptrSource^).rgbGreen := V;
                                 TRGBQuad(ptrSource^).rgbBlue  := B;
                           End
                           Else
                               Begin
                                    B := Round(TRGBQuad(ptrSource^).rgbBlue * ( 1 +  PourcB / 100 ));
                                    If B >= 255 Then
                                       TRGBQuad(ptrSource^).rgbBlue := 255
                                    Else
                                        If B <= 0 Then
                                           TRGBQuad(ptrSource^).rgbBlue := 0
                                        Else
                                            TRGBQuad(ptrSource^).rgbBlue := B;

                                    V := Round(TRGBQuad(ptrSource^).rgbGreen * ( 1 +  PourcV / 100 ));
                                    If V >= 255 Then
                                       TRGBQuad(ptrSource^).rgbGreen := 255
                                    Else
                                        If V <= 0 Then
                                           TRGBQuad(ptrSource^).rgbGreen := 0
                                        Else
                                            TRGBQuad(ptrSource^).rgbGreen := V;

                                    R := Round(TRGBQuad(ptrSource^).rgbRed * ( 1 +  PourcR / 100 ));
                                    If R >= 255 Then
                                       TRGBQuad(ptrSource^).rgbRed := 255
                                    Else
                                        If R <= 0 Then
                                           TRGBQuad(ptrSource^).rgbRed := 0
                                        Else
                                            TRGBQuad(ptrSource^).rgbRed := R;

                               End;

                               Inc( Integer(ptrSource), 4 );
                      End;

                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Y );
                 End;
            End;
     End;
End;

Procedure TFichierBitmap.Negatif;
{Objectif :Inverser chaque couleur du bitmap. Le principe est assez
           simple il suffit de soustraire la couleur d'un pixel  255.

                 Formule = 255 - Couleur

           Bien enttendu comme d'habitude si le bitmap dispose d'une palette
           il faut travailler avec directement...}
Var
   X, Y                   :Integer;  {Parcourir la palette ou les pixels}
   ptrBitmapOriginal      :Pointer;  {Pointeur sur les pixels du bitmap}
   ptrSource              :Pointer;  {Pointeur sur une ligne du bitmap}
   CouleurIndex           :Integer;  {Parcourir la palette du Bitmap}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Rcuprer un pointeur sur les Pixel du bitmap}
     PtrBitmapOriginal := pTabPixels;

      Case BmpINFO.biBitCount Of
      1, 4, 8:Begin
                   For CouleurIndex := 0 To PalNbCouleurs - 1 Do
                   Begin
                        ptrSource := Pointer( Cardinal(ptrDebutPal) + CouleurIndex * SizeOf(TRGBQuad) );
                        TRGBQuad(ptrSource^).rgbRed  := 255 - TRGBQuad(ptrSource^).rgbRed;
                        TRGBQuad(ptrSource^).rgbGreen:= 255 - TRGBQuad(ptrSource^).rgbGreen;
                        TRGBQuad(ptrSource^).rgbBlue := 255 - TRGBQuad(ptrSource^).rgbBlue;
                   End;
              End;

           16:Begin
                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             Word(ptrSource^) := Not Word(ptrSource^);
                             Inc( Integer(ptrSource), SizeOf(Word) );
                        End;
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Y );
                   End;
              End;

           24:Begin
                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             TRGBTriple(ptrSource^).rgbtRed :=  Not TRGBTriple(ptrSource^).rgbtRed;
                             TRGBTriple(ptrSource^).rgbtGreen :=  Not TRGBTriple(ptrSource^).rgbtGreen;
                             TRGBTriple(ptrSource^).rgbtBlue :=  Not TRGBTriple(ptrSource^).rgbtBlue;
                             Inc( Integer(ptrSource), SizeOf(TRGBTriple) );
                        End;
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Y );
                   End;
              End;

           32:Begin
                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             DWord(ptrSource^) := Not DWord(ptrSource^);
                             Inc( Integer(ptrSource), SizeOf(TRGBQuad) );
                        End;
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Y );
                   End;
              End;

           End;
End;

Procedure TFichierBitmap.Griser( PourcentClarity :Byte );
{Objectif :Rendre les couleurs du bitmap sous un tont de gris.
           Le ton de gris est donn selon le pourcentage reu en paramtre.
           Il existe un formule simple pour valuer ceci :

                      NouvelleCouleur = (R + V + B) Div 3

                      Rouge := NouvelleCouleur
                      Vert  := NouvelleCouleur
                      Bleu  := NouvelleCouleur
           }
Var
   X, Y                   :Integer;  {Parcourir la palette ou les pixels}
   ptrBitmapOriginal      :Pointer;  {Pointeur sur les pixels du bitmap}
   ptrSource              :Pointer;  {Pointeur sur une ligne du bitmap}
   CouleurIndex           :Integer;  {Parcourir la palette du Bitmap}
   NouvelleCouleur        :Word;     {Nouveau ton de gris obtenu}
   R, V, B                :Integer;  {Couleur temporaire pour extraction}
   NbBitAllumR,
   NbBitAllumV,
   NbBitAllumB            :Byte;     {Nombre de bits conscutif allums}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Vrifier les bornes de clarit. [0..100]}
     If PourcentClarity > 100 Then
        PourcentClarity := 100;

     {Rcuprer un pointeur sur les Pixel du bitmap}
     PtrBitmapOriginal := pTabPixels;

     Case BmpINFO.biBitCount Of
      1, 4, 8:Begin
                   For CouleurIndex := 0 To PalNbCouleurs - 1 Do
                   Begin
                        ptrSource := Pointer( Integer(ptrDebutPal) + CouleurIndex * SizeOf(TRGBQuad) );
                        NouvelleCouleur := Round( ( TRGBQuad(ptrSource^).rgbRed  * PourcentClarity  / 100 +
                                                    TRGBQuad(ptrSource^).rgbGreen * PourcentClarity / 100 +
                                                    TRGBQuad(ptrSource^).rgbBlue * PourcentClarity  / 100 ) / 3);

                        TRGBQuad(ptrSource^).rgbRed   := NouvelleCouleur;
                        TRGBQuad(ptrSource^).rgbGreen := NouvelleCouleur;
                        TRGBQuad(ptrSource^).rgbBlue  := NouvelleCouleur;
                   End;
              End;

           16:Begin
                  {Rcuprer la position du premier bits allume dans le Masque}
                   NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                   NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                   NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             If BitFieldBitmap Then
                             Begin
                                  R := (DWord(ptrSource^) And MasqueRouge) Shr NbBitAllumR;
                                  V := (DWord(ptrSource^) And MasqueVert) Shr NbBitAllumV;
                                  B := (DWord(ptrSource^) And MasqueBleu);

                                  NouvelleCouleur := Round( ( R * PourcentClarity / 100 +
                                                              V * PourcentClarity / 100 +
                                                              B * PourcentClarity / 100 ) / 3);

                                  {C'est la seul manire que j'ai trouver pour
                                   donner un ton de gris  une image. Car selon
                                   moi il faudrait beaucoup de calcul pour
                                   convertir un bitmap 16bpp en Gris car les
                                   les couleurs n'ont pas les mme limite.}
                                  NouvelleCouleur := (NouvelleCouleur Shl 11) Or (NouvelleCouleur Shl 6 ) Or NouvelleCouleur;
                             End
                             Else {5:5:5}
                                 Begin
                                      R := (DWord(ptrSource^) And $7C00 ) Shr 10;
                                      V := (DWord(ptrSource^) And $03E0 ) Shr 5;
                                      B := (DWord(ptrSource^) And $001F );

                                      NouvelleCouleur := Round( ( R * PourcentClarity / 100 +
                                                                  V * PourcentClarity / 100 +
                                                                  B * PourcentClarity / 100 ) / 3);

                                      NouvelleCouleur := (NouvelleCouleur Shl 10) Or (NouvelleCouleur Shl 5) Or NouvelleCouleur;
                                 End;

                             Word(ptrSource^) := NouvelleCouleur;
                             Inc( Integer(ptrSource), 2 );
                      End;
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Y );
                 End;
            End;

           24:Begin
                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             NouvelleCouleur := Round( ( TRGBTriple(ptrSource^).rgbtRed   * PourcentClarity / 100 +
                                                         TRGBTriple(ptrSource^).rgbtGreen * PourcentClarity / 100 +
                                                         TRGBTriple(ptrSource^).rgbtBlue  * PourcentClarity / 100 ) / 3 );

                             TRGBTriple(ptrSource^).rgbtRed   :=  NouvelleCouleur;
                             TRGBTriple(ptrSource^).rgbtGreen :=  NouvelleCouleur;
                             TRGBTriple(ptrSource^).rgbtBlue  :=  NouvelleCouleur;

                             Inc( Integer(ptrSource), SizeOf(TRGBTriple) );
                        End;
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Y );
                   End;
              End;

           32:Begin
                  {Rcuprer la position du premier bits allume dans le Masque}
                   NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                   NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                   NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                   Y := 0;
                   While Y < BmpINFO.biHeight Do
                   Begin
                        ptrSource := Pointer( Integer(PtrBitmapOriginal) );

                        For X := 0 To (BmpINFO.biWidth - 1) Do
                        Begin
                             If BitFieldBitmap Then
                             Begin
                                  R := (DWord(ptrSource^) And MasqueRouge) Shr NbBitAllumR;
                                  V := (DWord(ptrSource^) And MasqueVert) Shr NbBitAllumV;
                                  B := (DWord(ptrSource^) And MasqueBleu);

                                  NouvelleCouleur := Round( ( R * PourcentClarity / 100 +
                                                              V * PourcentClarity / 100 +
                                                              B * PourcentClarity / 100 ) / 3);

                                  TRGBQuad( ptrSource^ ).rgbRed   := NouvelleCouleur;
                                  TRGBQuad( ptrSource^ ).rgbGreen := NouvelleCouleur;
                                  TRGBQuad( ptrSource^ ).rgbBlue  := NouvelleCouleur;
                             End
                             Else
                                 Begin
                                      NouvelleCouleur := Round( ( TRGBQuad( ptrSource^ ).rgbRed   * PourcentClarity / 100 +
                                                                  TRGBQuad( ptrSource^ ).rgbGreen * PourcentClarity / 100 +
                                                                  TRGBQuad( ptrSource^ ).rgbBlue  * PourcentClarity / 100 ) / 3);

                                      TRGBQuad( ptrSource^ ).rgbRed   := NouvelleCouleur;
                                      TRGBQuad( ptrSource^ ).rgbGreen := NouvelleCouleur;
                                      TRGBQuad( ptrSource^ ).rgbBlue  := NouvelleCouleur;
                                 End;
                             Inc( Integer(ptrSource), SizeOf(TRGBQuad) );
                      End;
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Y );
                 End;
            End;
     End;
End;

Procedure TFichierBitmap.ConvertirEn24bpp;
{Objectif :Convertir le un bitmap en 24bpp.}

Var
   X, Y                     :Integer;   {Parcourir les pixels des Bitmaps}
   ptrBitmapOriginal,
   PtrBitmapCible           :Pointer;   {Pointeur sur le bitmap original et
                                         le nouveau.}
   pNouveauTableauPix       :Pointer;   {Pointeur sur un nouveau tableau de
                                         pixels}
   pOldRGBPixel             :Pointer;   {Pointeur sur une couleur de la palette
                                         du bitmap}
   ptrSurPalette            :Pointer;   {Pointeur sur la palette du bitmap 1bpp}
   pNewRGBPixel             :Pointer;   {Pointeur sur une pixel du nouveau
                                         bitmap}
   NouvelleLigne            :Integer;   {Nouvelle longueur d'une ligne dans
                                         un bitmap 1bpp}
   PixelBitPos              :Byte;      {Position de la pixel dans un octet}
   ShiftValue               :Byte;      {Dplacement pour aller rcuprer un bit}

   R, V, B                  :Integer;   {Couleur extraite d'un WORD}
   NbBitAllumR,
   NbBitAllumV,
   NbBitAllumB              :Byte;      {Nombre de bits conscutif allums}
   NbBitRouge,
   NbBitVert,
   NbBitBleu                :Byte;      {Position du premier bits allume}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Trouver la nouvelle taille du bitmap un fois convertit en 24bpp}
     NouvelleLigne := ((((((BmpINFO.biWidth * 24)+ 7) And $FFFFFFF8) Shr 3) + 3 ) And $FFFFFFFC);

     {Allouer de la mmoire pour un bitmap 24bpp}
     GetMem( pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

     {Rcuprer un pointeur sur les Pixels du bitmap 8bpp}
     PtrBitmapOriginal := pTabPixels;

     {Rcuprer un pointeur sur les Pixels du nouveau bitmap 24bpp}
     PtrBitmapCible := pNouveauTableauPix;

     {Commencer  lire la dernire ligne du bitmap}
     Y := 0;
     While Y < BmpINFO.biHeight Do
     Begin
          Case BmpINFO.biBitCount Of
               1:Begin
                      {Vrifier si le bitmap est bien un 1bpp}
                      If (BmpINFO.biBitCount <> 1) And (NbBitsParPixel <> 1) Then
                         Exit;

                      {Rcuprer un pointeur sur les Pixels du bitmap 1bpp}
                      PtrBitmapOriginal := Pointer(Integer(pTabPixels) + (Y * LongueurDuneLigne));
                      {Rcuprer un pointeur sur les Pixels du nouveau bitmap 24bpp}
                      PtrBitmapCible := Pointer(Integer(pNouveauTableauPix) + (Y * NouvelleLigne));

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                           {Calculer un pointeur sur une pixel du bitmap 1bpp}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + (X Shr 3) );

                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf(TRGBTriple) );

                           {Trouver la valeur correspondante dans la palette}
                           PixelBitPos := $80 Shr (X And 7);
                           ShiftValue := 7-(X And 7);
                           ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (( ( (PixelBitPos And Byte(pOldRGBPixel^)) Shr ShiftValue) * SizeOf(TRGBQuad) ) ) );

                           {Conversion de 1bpp  24bpp}
                           TRGBTriple( pNewRGBPixel^ ).rgbtRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBTriple( pNewRGBPixel^ ).rgbtGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                      End;
                 End;


               4:Begin
                      {Vrifier si le bitmap est bien un 4bpp}
                      If (BmpINFO.biBitCount <> 4) And (NbBitsParPixel <> 4) Then
                         Exit;

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                           {Pointeur sur les anciennes pixels}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + (X Shr 1) );

                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBTriple ) );

                           {Pointeur sur une couleur dans la palette}
                           If X Mod 2 <> 0 Then
                              ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (Byte( pOldRGBPixel^) And $0F) * SizeOf(TRGBQuad) )
                           Else
                               ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (Byte( pOldRGBPixel^) Shr 4) * SizeOf(TRGBQuad) );

                           {Conversion de 4bpp  24bpp}
                           TRGBTriple( pNewRGBPixel^ ).rgbtRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBTriple( pNewRGBPixel^ ).rgbtGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                      End;
                      {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Integer(PtrBitmapCible), NouvelleLigne );
                 End;


               8:Begin
                      {Vrifier si le bitmap est bien un 8bpp}
                      If (BmpINFO.biBitCount <> 8) And (NbBitsParPixel <> 8) Then
                         Exit;

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                          {Pointeur sur les anciennes pixels}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X );
                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBTriple ));

                           {Pointeur sur une couleur dans la palette}
                           ptrSurPalette := Pointer( Integer( PtrDebutPal ) + Byte(pOldRGBPixel^) * SizeOf(TRGBQuad) );

                           {Conversion de 8bpp  24bpp}
                           TRGBTriple( pNewRGBPixel^ ).rgbtRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBTriple( pNewRGBPixel^ ).rgbtGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                      End;
                      {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Integer(PtrBitmapCible), NouvelleLigne );
                 End;

               16 :Begin
                      {Vrifier si le bitmap est bien un 16bpp}
                      If (BmpINFO.biBitCount <> 16) And (NbBitsParPixel <> 16) Then
                         Exit;

                        {Rcuprer la position du premier bits allume dans le Masque}
                        NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                        NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                        NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                        {Rcuprer le nombre de bits conccutif allums}
                        NbBitRouge := RecupNbBitsAllumer( MasqueRouge);
                        NbBitVert  := RecupNbBitsAllumer( MasqueVert);
                        NbBitBleu  := RecupNbBitsAllumer( MasqueBleu);

                        For X := 0 To (BmpINFO.biWidth -1) Do
                        Begin
                             {Pointeur sur les anciennes pixels}
                             pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X * SizeOf( Word ));
                             {Pointeur sur les nouvelles pixels}
                             pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBTriple ));

                             If BitFieldBitmap Then
                             Begin
                                  R := (DWord(pOldRGBPixel^) And MasqueRouge) Shr NbBitAllumR;
                                  V := (DWord(pOldRGBPixel^) And MasqueVert) Shr NbBitAllumV;
                                  B := (DWord(pOldRGBPixel^) And MasqueBleu);

                                  {Conversion de 16bpp  24bpp. Il faut rvaluer les couleurs car
                                  elles ne sont pas dans l'intervalle 0  255 et cela cause une
                                  image trs fade un fois convertie.}
                                  TRGBTriple( pNewRGBPixel^ ).rgbtRed   := Round( R * 255 / ((1 Shl NbBitRouge) - 1));
                                  TRGBTriple( pNewRGBPixel^ ).rgbtGreen := Round( V * 255 / ((1 Shl NbBitVert) - 1));
                                  TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := Round( B * 255 / ((1 Shl NbBitBleu) - 1));
                             End
                             Else {5:5:5}
                                  Begin
                                  R := (DWord(pOldRGBPixel^) And $7C00 ) Shr 10;
                                  V := (DWord(pOldRGBPixel^) And $03E0 ) Shr 5;
                                  B := (DWord(pOldRGBPixel^) And $001F );

                                  {Conversion de 16bpp  24bpp. Il faut rvaluer les couleurs car
                                  elles ne sont pas dans l'intervalle 0  255 et cela cause une
                                  image trs fade un fois convertie.}
                                  TRGBTriple( pNewRGBPixel^ ).rgbtRed   := Round( R * 255 / 31);
                                  TRGBTriple( pNewRGBPixel^ ).rgbtGreen := Round( V * 255 / 31);;
                                  TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := Round( B * 255 / 31);;
                             End;
                        End;
                        {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Integer(PtrBitmapCible), NouvelleLigne );
                  End;

               32:Begin
                      {Vrifier si le bitmap est bien un 32bpp}
                      If (BmpINFO.biBitCount <> 32) And (NbBitsParPixel <> 32) Then
                         Exit;

                       For X := 0 To (BmpINFO.biWidth -1) Do
                       Begin
                            {Pointeur sur les anciennes pixels}
                            pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X * SizeOf( TRGBQuad ));
                            {Pointeur sur les nouvelles pixels}
                            pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBTriple ));

                            {Conversion de 32bpp  24bpp}
                            TRGBTriple( pNewRGBPixel^ ).rgbtRed   := TRGBQuad(pOldRGBPixel^).rgbRed;
                            TRGBTriple( pNewRGBPixel^ ).rgbtGreen := TRGBQuad(pOldRGBPixel^).rgbGreen;
                            TRGBTriple( pNewRGBPixel^ ).rgbtBlue  := TRGBQuad(pOldRGBPixel^).rgbBlue;
                       End;
                       {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                       Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                       Inc( Integer(PtrBitmapCible), NouvelleLigne );
                  End;

                 {Si le bpp du bitmap n'et pas reconnu alors quitter la procdure}
                 Else
                     Exit;
         End;
          {Incrmente le compteur}
          Inc( Y );
     End;

    {Rallouer la mmoire de l'ancien tableau de pixel.}
     FreeMem( pTabPixels, DimTableauPixels );
     GetMem( pTabPixels, BmpINFO.biHeight * NouvelleLigne );

    {Recopi le tout dans le tableau de pixel original}
     CopyMemory( pTabPixels, pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

    {Rcuprer le pointeur plublique sur le nouveau tableau de pixels}
     pPixels := pTabPixels;

    {Librer la mmoire du tableau de pixel temporaire}
     FreeMem( pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

    {Affectation des nouveaux paramtres du bitmap}
     BmpINFOFichier.bfOffBits := SizeOf(TBITMAPFILEHEADER) + SizeOf(TBITMAPINFOHEADER);
     BmpINFOFichier.bfSize := SizeOf( TBITMAPFILEHEADER ) + SizeOf(TBITMAPINFOHEADER)
                             + DimTableauPixels;

     TBITMAPINFO(pBmpINFO^).bmiHeader.biBitCount := 24;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biCompression := BI_RGB;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biClrUsed := 0;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biClrImportant := 0;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biSize := SizeOf( TBITMAPINFOHEADER );

     BmpINFO.bibitCount := 24;
     BmpINFO.biCompression := BI_RGB;
     BitFieldBitmap := False;
     BmpINFO.biClrUsed := 0;
     BmpINFO.biClrImportant := 0;
     BmpINFO.biSize := SizeOf( TBITMAPINFOHEADER );

     NbBitsParPixel := 24;
     PalNbCouleurs := 0;
     GrosseurPal := 0;
     LongueurDuneLigne := (((((BmpINFO.biWidth * BmpINFO.biBitCount)+ 7) And $FFFFFFF8) Shr 3) + 3 ) And $FFFFFFFC;
     DimTableauPixels := BmpINFO.biHeight * NouvelleLigne;
End;


Procedure TFichierBitmap.ConvertirEn32bpp( Alpha :Byte );
{Objectif :Convertir le un bitmap en 24bpp.}

Var
   X, Y                     :Integer;   {Parcourir les pixels des Bitmaps}
   ptrBitmapOriginal,
   PtrBitmapCible           :Pointer;   {Pointeur sur le bitmap original et
                                         le nouveau.}
   pNouveauTableauPix       :Pointer;   {Pointeur sur un nouveau tableau de
                                         pixels}
   pOldRGBPixel             :Pointer;   {Pointeur sur une couleur de la palette
                                         du bitmap}
   ptrSurPalette            :Pointer;   {Pointeur sur la palette du bitmap X bpp}
   pNewRGBPixel             :Pointer;   {Pointeur sur une pixel du nouveau
                                         bitmap}
   NouvelleLigne            :Integer;   {Nouvelle longueur d'une ligne dans
                                         un bitmap X bpp}
   PixelBitPos              :Byte;      {Position de la pixel dans un octet}
   ShiftValue               :Byte;      {Dplacement pour aller rcuprer un bit}

   R, V, B                  :Integer;   {Couleur extraite d'un WORD}
   NbBitAllumR,
   NbBitAllumV,
   NbBitAllumB              :Byte;      {Nombre de bits conscutif allums}
   NbBitRouge,
   NbBitVert,
   NbBitBleu                :Byte;      {Position du premier bits allume}
Begin
     {Vrifier si le bitmap est belle et bien charg en mmoire}
     If (Not BitmapCharger) And (pTabPixels = Nil) Then
         Exit;

     {Trouver la nouvelle taille du bitmap un fois convertit en 24bpp}
     NouvelleLigne := ((((((BmpINFO.biWidth * 32)+ 7) And $FFFFFFF8) Shr 3) + 3 ) And $FFFFFFFC);

     {Allouer de la mmoire pour un bitmap 24bpp}
     GetMem( pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

     {Rcuprer un pointeur sur les Pixels du bitmap 8bpp}
     PtrBitmapOriginal := pTabPixels;

     {Rcuprer un pointeur sur les Pixels du nouveau bitmap 24bpp}
     PtrBitmapCible := pNouveauTableauPix;

     {Commencer  lire la dernire ligne du bitmap}
     Y := 0;
     While Y < BmpINFO.biHeight Do
     Begin
          Case BmpINFO.biBitCount Of
               1:Begin
                      {Vrifier si le bitmap est bien un 1bpp}
                      If (BmpINFO.biBitCount <> 1) And (NbBitsParPixel <> 1) Then
                         Exit;

                      {Rcuprer un pointeur sur les Pixels du bitmap 1bpp}
                      PtrBitmapOriginal := Pointer(Integer(pTabPixels) + (Y * LongueurDuneLigne));
                      {Rcuprer un pointeur sur les Pixels du nouveau bitmap 24bpp}
                      PtrBitmapCible := Pointer(Integer(pNouveauTableauPix) + (Y * NouvelleLigne));

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                           {Calculer un pointeur sur une pixel du bitmap 1bpp}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + (X Shr 3) );

                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf(TRGBQuad) );

                           {Trouver la valeur correspondante dans la palette}
                           PixelBitPos := $80 Shr (X And 7);
                           ShiftValue := 7-(X And 7);
                           ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (( ( (PixelBitPos And Byte(pOldRGBPixel^)) Shr ShiftValue) * SizeOf(TRGBQuad) ) ) );

                           {Conversion de 1bpp  32bpp}
                           TRGBQuad( pNewRGBPixel^ ).rgbRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBQuad( pNewRGBPixel^ ).rgbGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBQuad( pNewRGBPixel^ ).rgbBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                           TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                      End;
                 End;


               4:Begin
                      {Vrifier si le bitmap est bien un 4bpp}
                      If (BmpINFO.biBitCount <> 4) And (NbBitsParPixel <> 4) Then
                         Exit;

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                           {Pointeur sur les anciennes pixels}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + (X Shr 1) );

                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBQuad ) );

                           {Pointeur sur une couleur dans la palette}
                           If X Mod 2 <> 0 Then
                              ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (Byte( pOldRGBPixel^) And $0F) * SizeOf(TRGBQuad) )
                           Else
                               ptrSurPalette := Pointer( Integer( PtrDebutPal ) + (Byte( pOldRGBPixel^) Shr 4) * SizeOf(TRGBQuad) );

                           {Conversion de 4bpp  32bpp}
                           TRGBQuad( pNewRGBPixel^ ).rgbRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBQuad( pNewRGBPixel^ ).rgbGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBQuad( pNewRGBPixel^ ).rgbBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                           TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                      End;
                      {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Integer(PtrBitmapCible), NouvelleLigne );
                 End;


               8:Begin
                      {Vrifier si le bitmap est bien un 8bpp}
                      If (BmpINFO.biBitCount <> 8) And (NbBitsParPixel <> 8) Then
                         Exit;

                      For X := 0 To (BmpINFO.biWidth -1) Do
                      Begin
                          {Pointeur sur les anciennes pixels}
                           pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X );
                           {Pointeur sur les nouvelles pixels}
                           pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBQuad ));

                           {Pointeur sur une couleur dans la palette}
                           ptrSurPalette := Pointer( Integer( PtrDebutPal ) + Byte(pOldRGBPixel^) * SizeOf(TRGBQuad) );

                           {Conversion de 8bpp  32bpp}
                           TRGBQuad( pNewRGBPixel^ ).rgbRed   := TRGBQuad(ptrSurPalette^).rgbRed;
                           TRGBQuad( pNewRGBPixel^ ).rgbGreen := TRGBQuad(ptrSurPalette^).rgbGreen;
                           TRGBQuad( pNewRGBPixel^ ).rgbBlue  := TRGBQuad(ptrSurPalette^).rgbBlue;
                           TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                      End;
                      {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                      Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                      Inc( Integer(PtrBitmapCible), NouvelleLigne );
                 End;

               16 :Begin
                      {Vrifier si le bitmap est bien un 16bpp}
                      If (BmpINFO.biBitCount <> 16) And (NbBitsParPixel <> 16) Then
                         Exit;

                        {Rcuprer la position du premier bits allume dans le Masque}
                        NbBitAllumR := RecupPos1BitAllumer( MasqueRouge );
                        NbBitAllumV := RecupPos1BitAllumer( MasqueVert );
                        NbBitAllumB := RecupPos1BitAllumer( MasqueBleu );

                        {Rcuprer le nombre de bits conccutif allums}
                        NbBitRouge := RecupNbBitsAllumer( MasqueRouge);
                        NbBitVert  := RecupNbBitsAllumer( MasqueVert);
                        NbBitBleu  := RecupNbBitsAllumer( MasqueBleu);

                        For X := 0 To (BmpINFO.biWidth -1) Do
                        Begin
                             {Pointeur sur les anciennes pixels}
                             pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X * SizeOf( Word ));
                             {Pointeur sur les nouvelles pixels}
                             pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBQuad ));

                             If BitFieldBitmap Then
                             Begin
                                  R := (DWord(pOldRGBPixel^) And MasqueRouge) Shr NbBitAllumR;
                                  V := (DWord(pOldRGBPixel^) And MasqueVert) Shr NbBitAllumV;
                                  B := (DWord(pOldRGBPixel^) And MasqueBleu);

                                  {Conversion de 16bpp  32bpp. Il faut rvaluer les couleurs car
                                  elles ne sont pas dans l'intervalle 0  255 et cela cause une
                                  image trs fade un fois convertie.}
                                  TRGBQuad( pNewRGBPixel^ ).rgbRed   := Round( R * 255 / ((1 Shl NbBitRouge) - 1));
                                  TRGBQuad( pNewRGBPixel^ ).rgbGreen := Round( V * 255 / ((1 Shl NbBitVert) - 1));
                                  TRGBQuad( pNewRGBPixel^ ).rgbBlue  := Round( B * 255 / ((1 Shl NbBitBleu) - 1));
                                  TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                             End
                             Else {5:5:5}
                                  Begin
                                  R := (DWord(pOldRGBPixel^) And $7C00 ) Shr 10;
                                  V := (DWord(pOldRGBPixel^) And $03E0 ) Shr 5;
                                  B := (DWord(pOldRGBPixel^) And $001F );

                                  {Conversion de 16bpp  32bpp. Il faut rvaluer les couleurs car
                                  elles ne sont pas dans l'intervalle 0  255 et cela cause une
                                  image trs fade un fois convertie.}
                                  TRGBQuad( pNewRGBPixel^ ).rgbRed   := Round( R * 255 / 31);
                                  TRGBQuad( pNewRGBPixel^ ).rgbGreen := Round( V * 255 / 31);;
                                  TRGBQuad( pNewRGBPixel^ ).rgbBlue  := Round( B * 255 / 31);;
                                  TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                             End;
                        End;
                        {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                        Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                        Inc( Integer(PtrBitmapCible), NouvelleLigne );
                  End;

               24:Begin
                      {Vrifier si le bitmap est bien un 24bpp}
                      If (BmpINFO.biBitCount <> 24) And (NbBitsParPixel <> 24) Then
                         Exit;

                       For X := 0 To (BmpINFO.biWidth -1) Do
                       Begin
                            {Pointeur sur les anciennes pixels}
                            pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X * SizeOf( TRGBTriple ));
                            {Pointeur sur les nouvelles pixels}
                            pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBQuad ));

                            {Conversion de 24bpp  32bpp}
                           TRGBQuad( pNewRGBPixel^ ).rgbRed   := TRGBTriple(pOldRGBPixel^).rgbtRed;
                           TRGBQuad( pNewRGBPixel^ ).rgbGreen := TRGBTriple(pOldRGBPixel^).rgbtGreen;
                           TRGBQuad( pNewRGBPixel^ ).rgbBlue  := TRGBTriple(pOldRGBPixel^).rgbtBlue;
                           TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;
                        End;
                       {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                       Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                       Inc( Integer(PtrBitmapCible), NouvelleLigne );
                  End;

               32:Begin
                      {Si le format est dj un 32bpp alors offrir  l'usager
                       la possibilit de modifier la valeur Alpha du Bitmap}

                      {Vrifier si le bitmap est bien un 32bpp}
                      If (BmpINFO.biBitCount <> 32) And (NbBitsParPixel <> 32) Then
                         Exit;

                       For X := 0 To (BmpINFO.biWidth -1) Do
                       Begin
                            {Pointeur sur les anciennes pixels}
                            pOldRGBPixel := Pointer( Integer( PtrBitmapOriginal ) + X * SizeOf( TRGBQuad ));
                            {Pointeur sur les nouvelles pixels}
                            pNewRGBPixel := Pointer( Integer( PtrBitmapCible ) + X * SizeOf( TRGBQuad ));

                            {Conversion de 32bpp  24bpp}
                            TRGBQuad( pNewRGBPixel^ ).rgbRed   := TRGBQuad(pOldRGBPixel^).rgbRed;
                            TRGBQuad( pNewRGBPixel^ ).rgbGreen := TRGBQuad(pOldRGBPixel^).rgbGreen;
                            TRGBQuad( pNewRGBPixel^ ).rgbBlue  := TRGBQuad(pOldRGBPixel^).rgbBlue;

                            TRGBQuad( pNewRGBPixel^ ).rgbReserved := Alpha;  {Affectation de la valeur Alpha}
                       End;
                       {Avanc en hauteur, d'une ligne (Les deux bitmaps)}
                       Inc( Integer(PtrBitmapOriginal), LongueurDuneLigne );
                       Inc( Integer(PtrBitmapCible), NouvelleLigne );
                  End;


                 {Si le bpp du bitmap n'et pas reconnu alors quitter la procdure}
                 Else
                     Exit;
         End;
          {Incrmente le compteur}
          Inc( Y );
     End;

    {Rallouer la mmoire de l'ancien tableau de pixel.}
     FreeMem( pTabPixels, DimTableauPixels );
     GetMem( pTabPixels, BmpINFO.biHeight * NouvelleLigne );

    {Recopi le tout dans le tableau de pixel original}
     CopyMemory( pTabPixels, pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

    {Rcuprer le pointeur plublique sur le nouveau tableau de pixels}
     pPixels := pTabPixels;

    {Librer la mmoire du tableau de pixel temporaire}
     FreeMem( pNouveauTableauPix, BmpINFO.biHeight * NouvelleLigne );

    {Affectation des nouveaux paramtres du bitmap}
     BmpINFOFichier.bfOffBits := SizeOf(TBITMAPFILEHEADER) + SizeOf(TBITMAPINFOHEADER);
     BmpINFOFichier.bfSize := SizeOf( TBITMAPFILEHEADER ) + SizeOf(TBITMAPINFOHEADER)
                             + DimTableauPixels;

     TBITMAPINFO(pBmpINFO^).bmiHeader.biBitCount := 32;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biCompression := BI_RGB;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biClrUsed := 0;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biClrImportant := 0;
     TBITMAPINFO(pBmpINFO^).bmiHeader.biSize := SizeOf( TBITMAPINFOHEADER );

     BmpINFO.bibitCount := 32;
     BmpINFO.biCompression := BI_RGB;
     BitFieldBitmap := False;
     BmpINFO.biClrUsed := 0;
     BmpINFO.biClrImportant := 0;
     BmpINFO.biSize := SizeOf( TBITMAPINFOHEADER );

     NbBitsParPixel := 32;
     PalNbCouleurs := 0;
     GrosseurPal := 0;
     LongueurDuneLigne := (((((BmpINFO.biWidth * BmpINFO.biBitCount)+ 7) And $FFFFFFF8) Shr 3) + 3 ) And $FFFFFFFC;
     DimTableauPixels := BmpINFO.biHeight * NouvelleLigne;
End;

end.
